/*
 * $Id$
 *
 * Copyright (c) 2014, Juniper Networks, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see
 * <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 */


#ifndef _ATAVENDOR_DB_H_
#define _ATAVENDOR_DB_H_

#include <sys/types.h>
#include <sys/ata.h>

#include "atacmds.h"

#define VENDOR_ATTRIB_LEN 25
#define STATUS_FLAG   0
#define NORMAL_VALUE  1
#define WORST_VALUE   2
#define RAW_VALUE     3

struct smart_ssd_thresholds;
typedef int (*vendorchecksmart_t) (struct ata_smart_values,
				   struct smart_ssd_thresholds);

struct ata_vendor_attributes {
    unsigned char id;
    unsigned char vendor_attrib_name[VENDOR_ATTRIB_LEN];
} __attribute__((packed));

struct ata_vendor_db {
    const char *vendor_series_name;
    const struct ata_vendor_attributes *vendor_attributes;
    vendorchecksmart_t checksmart;	
} __attribute__((packed));

/* The data fields of unigen smart attribute structure are not same
 * as the struct ata_smart_attribute.
 * The attribute Structure for the Unigen SSD's is in the following
 * format:
 * 0 Attribute ID Number
 * 1:2 Status Flag
 * 3:8 Raw Data
 * 9 Value
 * 10 Worst
 * 11 Reserved
 * so the only the data field position of the struct ata_smart_attribute_unigen
 * are changed from struct ata_smart_attribute.
 */
struct ata_smart_attribute_unigen {
    unsigned char id;
    unsigned short status_flag;
    /* note char array used since linux can
     * not handle sizeof short correctly */
    unsigned char raw[6];
    unsigned char normalized;
    unsigned char worse_normal;
    unsigned char reserv;
} __attribute__((packed));

/* struct is for ssd threshold entries which are updated from smartd.conf */
struct smart_ssd_threshold_entry {
    unsigned char id;
    unsigned int threshold;
    unsigned char flag;
#define FlAG_VALID    (1 << 7)  /* Bit to indicate the flag is valid */
#define FLAG_RAW      (1 << 0)  /* Bit to indicate the Raw value */
#define FLAG_NORM     (0 << 0)  /* Bit for normalized value.same bit as raw */
#define FLAG_LOW      (1 << 1)  /* Bit to check against low value */
#define FLAG_HIGH     (0 << 1)  /* Bit for high value. same bit as low value */
} __attribute__((packed));

/* this for the ssd thresholds entries */
struct smart_ssd_thresholds {
    unsigned char isdevicessd;
    struct smart_ssd_threshold_entry thres_entries[NUMBER_ATA_SMART_ATTRIBUTES];
    unsigned char ssd_model[40];
} __attribute__((packed));

#define IS_FLAG_VALID(entry, id) (entry.thres_entries[id].flag & FlAG_VALID)
#define IS_FLAG_RAW(entry, id)   (entry.thres_entries[id].flag & FLAG_RAW)
#define IS_FLAG_LOW(entry, id)   (entry.thres_entries[id].flag & FLAG_LOW)

int isvendorattribexists(void);
void atavendorattributesdbupdate(char *model);
char *atagetvendorattribname (unsigned char id);
int isunigenattributetbl(void);
int printsmartvalues(void *ptr, int smartdesc, int pos);
int ssdchecksmart(struct ata_smart_values data,
		  struct smart_ssd_thresholds thres);

#endif
